/*
 * Copyright (c) 2012, Noel Lemouel
 * All rights reserved.
 */

#include <sysutil.h>
#include <drivers/i2c-gpio/i2c-gpio.h>

#include "tmp275.h"

#define TMP275_I2C_ADDR 0x90

int tmp275_echo()
{
   int i;

   i = 0;

   i2c_start();
   i=i2c_set_ctrl(TMP275_I2C_ADDR, I2C_CTRL_WR);
   i2c_stop();

   if(i)
   {
      printf("tmp275: [echo] i2c link error \n");
   }else{
      printf("tmp275: [echo] i2c link OK \n");
   }

   return i;
}

unsigned short tmp275_read(unsigned char reg)
{
   int i;
   unsigned short val = 0x0000;

   i = 0;

   i2c_start();  
   i |= i2c_set_ctrl(TMP275_I2C_ADDR, I2C_CTRL_WR);
   i |= i2c_write_byte(reg);
   i2c_start();
   i |= i2c_set_ctrl(TMP275_I2C_ADDR, I2C_CTRL_RD);
   if(reg != 0x01) val |= (i2c_read_byte(I2C_ACK) << 8);    /* reg_0x01 is only 8bits wide */
   val |= i2c_read_byte(I2C_ACK);
   i2c_stop();

   if(i)
   {
      printf("tmp275: [read] i2c link error\n");
   }else{
      // printf("tmp275: [read] i2c link OK\n");
   }

   return val;
}

/* hex to Celsius */
int tmp275_xtoc(unsigned char tr0, unsigned char tr1, tmp275_tp_t* tp)
{
   unsigned int f;

   tp->u = 'c';
   tp->s = (tr0 & 0x80);

   tr1 >>= 4;
   f = tr1 * 625;

   if(!tp->s){
      tp->i = tr0;
      tp->f = f;
   }else{
      tp->i = ~(tr0 - 1);
      tp->f = ~(tr1 - 1);
   }

   return 0;
}

/* hex to Fahrenheit */
int tmp275_xtof(unsigned char tr0, unsigned char tr1, tmp275_tp_t* tp)
{
   /* use integer only / tp_f = ((5/9)*tp_c)+32 */
   /* integer is expented to be at least 16bits wide */
   int i;
   unsigned int r;
   unsigned int f;
   unsigned int s;

   tmp275_xtoc(tr0, tr1, tp);

   tp->u = 'f';
   i = tp->i;
   f = tp->f;
   s = tp->s;

   /* multiply by 1.8 */
   i*=18;
   tp->i = i/10;
   r = i - (tp->i*10);  /* remaining float portion (0/2/4/6/8) */

   f/=10;
   f*=18;
   if(f > 10000){
      tp->f = f-10000;
      tp->i += 1;
   }else{
      tp->f = f;
   }

   f = tp->f;
   r *= 1000;

   if((f+r) > 10000){
      tp->i += 1;
      tp->f = f+r-10000;
   }else{
      tp->f += r;
   }

   /* sign */
   tp->s = (s && (tp->i>=32)) ? (s && (tp->i==32) && (!tp->f)) ? 0 : 1 : 0;

   /* add 32 */
   i = tp->i;
   if(s){
      i = -i;
      if(!tp->s){
         if(tp->f) tp->f = 10000 - tp->f;
         tp->i = tp->s ? -(i+31) : i+31;
      }else{
         tp->i = (s && tp->s) ? -(i+32) : i+32;
      }
   }else{
      tp->i += 32;
   }

   return 0;
}

